<?php
/*******************************************************************************
 * T E M P L A T E   S Y S T E M
 *
 * 	Antoni Jakubuak <antek@sk8.pl>
 * 	(c) Firma "JAKUBIAK" 2004
 * 	http://www.jakubiak.biz/
 *
 * 	Yet Another Template Engine but I use Polish notation 
 * (or reverese Polish notation) for parsing data. 
 *
 * >>Reverse Polish notation (RPN) (aka postfix notation) is an arithmetic formula 
 * notation, derived from the polish notation introduced in 1920 by the Polish 
 * mathematician Jan ukasiewicz. RPN was invented by Australian philosopher 
 * and computer scientist Charles Hamblin in the mid-1950s, 
 * to enable zero-address memory stores.<<
 * (http://en.wikipedia.org/wiki/Reverse_Polish_Notation)
 *
 * TODO: copy example to documentation:
 *
 * Why Polish Notation:
 *	- it's fast 
 *	- do not need compilatin
 *	- it has functionality of programing language
 * 	- it's extramly easy to parse
 *
 * Example 1:
 * <test.html>
 *	{get a}
 * <test.php>
 *	$t = new Template;
 *	$t->assing( 'a', 'Hello World' );
 *	$t->fetch( 'test.html' );
 * <result>
 *	Hello World
 * 
 * Examlpe 2:
 * <test2.html>
 * 	<table>
 *		{foreach get a get row item key}
 *		<!-- BEGIN row -->
 *		<tr>
 * 			<td>{get key}</td>
 *			<td>{get item}</td>
 *			<td>{+ get item 10}</td>
 *		</tr>
 *		<!-- END row -->
 *	</table>
 * <test2.php>
 *	$t = new Template;
 *	$t->assing( 'a', range( 10, 12 ) );
 *	$t->fetch( 'test2.html' );
 * <result retuns html table containg information>
 * 	0	10	20
 *	1	11	21
 *	2	12	22
 *	
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *
 * Variable declaration:
 *	Like PHPLIB but do not need calling set_block
 * <!-- BEGIN label -->
 * <!-- END label -->
 *	
 * Executing:
 * {}
 *
 *
 * $Id: template.inc.php,v 1.7 2004/06/27 14:11:35 spiacy Exp $
 */

class Template {
	/*
	** Use or not reverse Polish notation
	*/
	var $reverse_Polish_notation = false;

	/*
	** Max depth in recursion parsing
	*/
	var $depth = 50;

	/*
	** Scika do otwieranych plikw
	*/
	var $root = './';
	
	/*
	** Variables
	*/
	var $vars = array(
		'NULL'		=> null,
	);

	/*
	** Functions
	*/
	var $functions = array(
		// basic operators
		'get'		=> array( 1, '_call_get' ),
		'parse'		=> array( 1, '_call_parse' ),

		// loops
		'foreach'	=> array( 4, '_call_foreach' ),
		// 
		'default'	=> array( 2, '_call_default' ),
		'if'		=> array( 3, '_call_if' ),
		// mathematical
		'+'		=> array( 2, '_call_plus' ),
		'-'		=> array( 2, '_call_minus' ),
		'*'		=> array( 2, '_call_mul' ),
		'/'		=> array( 2, '_call_div' ),
		// logical
		'=='		=> array( 2, '_call_eq' ),
		'!='		=> array( 2, '_call_neq' ),
		'<'		=> array( 2, '_call_lt' ),
		'>'		=> array( 2, '_call_gt' ),
		'<='		=> array( 2, '_call_lteq' ),
		'>='		=> array( 2, '_call_gteq' ),
		// aliases for logical 
		'eq'		=> array( 2, '_call_eq' ),
		'neq'		=> array( 2, '_call_neq' ),
		'lt'		=> array( 2, '_call_lt' ),
		'gt'		=> array( 2, '_call_gt' ),
		'lteq'		=> array( 2, '_call_lteq' ),
		'gteq'		=> array( 2, '_call_gteq' ),

		// bardzo przydatny html_options
		'html_options'	=> array( 3, '_call_html_options' ),

		// escaping from smarty
		'escape'	=> array( 2, '_call_escape' ),

		// PHP wrappers
		'trim'		=> array( 1, '_call_trim' ), 
		'count'		=> array( 1, '_call_count' ),

	);
	
	


	function Template( $root = './' ) {
		$this->root = $root;
	}


	/*
	** Funkcje dla kompatybilnoci z SMARTY
	*/
	function assign( $varname, $value ) { 
		$this->set_var( $varname, $value ); 
	}
	function fetch( $filename ) {
		$block = uniqid( time() );
		$this->set_file( $block, $filename );
		return $this->get( $block );
	}
		

	
	

	/*
	** Wczytanie plikw
	*/
	function set_file( $varname, $filename = '' ) {
		if ( is_array( $varname ) ) {
			foreach( $varname as $k => $v ) $this->set_file( $k, $v );
			return;
		}
		$this->set_block( $varname, file_get_contents( $this->root . $filename ) );
	}

	/*
	** Wyszukanie blokw i wpakowanie ich do zmiennych
	** Przypisanie tego co zostanie do zmiennej
	*/
	function set_block( $varname, $value = '' ) {
		if ( is_array( $varname ) ) {
			foreach( $varname as $k => $v ) $this->set_block( $k, $v );
			return;
		}
		$value = preg_replace_callback( 
			'/<!--\s+BEGIN\s+([a-zA-Z0-9_]+)\s+-->(.*?)<!--\s+END\s+\1\s+-->/sm',
			array( &$this, '_set_callback' ),
			$value
		);
		$this->vars[ $varname ] = $value;
	}
	/*
	** Przypisanie zmiennych
	*/
	function set_var( $varname, $value = '' ) {
		if ( is_array( $varname ) ) {
			foreach( $varname as $k => $v ) $this->set_var( $k, $v );
			return;
		}
		$this->vars[ $varname ] = $value;
	}

	/*
	** Parsowanie zmiennej i pobranie wartoci
	*/
	function get( $varname ) {
		$tokens = array( $varname, 'get', 'parse' );
		if ( $this->reverse_Polish_notation ) {
			$tokens = array( $varname, 'get', 'parse' );
		} else {
			$tokens = array( 'parse', 'get', $varname );
		}
		$ret = $this->_get( $tokens );
		return array_pop( $ret );
	}


	/*
	** Przetwaanie stosu:
	** funkcja powinna zwraca tablic 1 elementow
	*/
	function _get( $tokens = array() ) {
		$stack = array();

		if ( $this->reverse_Polish_notation == false ) {
			$tokens = array_reverse( $tokens );
		}
	
		foreach( $tokens as $token ) {
			switch( true ) {
				case is_numeric( $token ):
					// liczba
					array_push( $stack, $token );
					break;
				case isset( $this->functions[ $token ] );
					// funkcja
					$params = array();
					for ( $i = 0; $i < $this->functions[ $token ][ 0 ]; $i++ ) {
						$params[] = array_pop( $stack );
					}
					$ret = $this->_call( $token, $params );
					array_push( $stack, $ret );
					break;
				default:
					// etykieta
					array_push( $stack, $token );
			}
		}
		return $stack;
	}


	



	
	
	/*
	** Zapisanie znalezionego bloku do zmiennej i usunicie bloku
	*/
	function _set_callback( $matches ) {
		$this->set_block( $matches[1], $matches[2] );
		return '';
	}

	/*
	** Podstawienie
	*/
	function _get_callback( $matches ) {
		$tokens = split( ' ', $matches[1] );
		$ret = $this->_get( $tokens );
		$val = array_pop( $ret );
		return $val;
	}

	
	/*
	** Pobranie wartoci zmiennej
	*/
	function _get_var( $varname ) {
		// zmienna moe by tabli lub obiektem
		$parts = split( '\.', $varname );
		$ret = $this->vars;
		foreach( $parts as $key ) {
			switch( true ) {
				case is_array( $ret )	: $ret = isset( $ret[ $key ] ) ? $ret[ $key ] : ''; break;
				case is_object( $ret )	: $ret = isset( $ret->$key ) ? $ret->key : ''; break;
				default:
			}
		}
		return $ret;
	}




	
	/*
	** Wykonanie instrukcji
	*/
	function _call( $function, $params = array() ) {
		$call = $this->functions[ $function ][ 1 ];
		if ( method_exists( $this, $call ) ) {
			$ret = call_user_func_array( array( &$this, $call ), $params );
		} elseif ( function_exists( $call ) ) {
			$ret = call_user_func_array( $call, $params );
		} else {
			trigger_error( "Call undefined function $call", E_USER_ERROR );
			$ret = null;
		}
		return $ret;
	}

	/*
	** Declared functions:
	*/


	/*
	** Getting a value of variable assinged with function set_var or set_block
	*/
	function _call_get( $label ) {
		return $this->_get_var( $label );
	}

	/*
	** Internal parsing a string
	** Searching of {...} and executing stack
	*/
	function _call_parse( $string ) {
		$this->depth--;
		if ( $this->depth == 0 ) {
			$ret = $string;
		} else {
			$ret = preg_replace_callback( 
				//				'/{([_$a-zA-Z0-9][^}]*)}/', 
				'/{([^}]+)}/', 
				array( &$this, '_get_callback' ), 
				$string
			);
		}
		$this->depth++;
		return $ret;
	}

	

	/*
	** foreach loop
 	**	loops all ellement of 'array'. set variable 'item' and 'key'
	**	returns concaneted parsed 'string'
	*/
	function _call_foreach( $array, $string, $item, $key ) {
		$ret = '';
		if ( is_array( $array ) ) foreach( $array as $k => $v ) {
			// przypisanie nowych zmiennych do szablonu
			$this->set_var( $item, $v );
			// jeeli jest klucz	
			$this->set_var( $key, $k );
			$ret .= $this->_call_parse( $string );
		}
		return $ret;
	}

	/*
	** Math functions
	*/
	function _call_plus( $a, $b )	{ return $a + $b; }
	function _call_minus( $a, $b )	{ return $a - $b; }
	function _call_mul( $a, $b )	{ return $a * $b; }
	function _call_div( $a, $b )	{ return $a / $b; }

	/*
	** Equations
	*/
	function _call_eq( $a, $b )	{ return $a == $b; }
	function _call_neq( $a, $b )	{ return $a != $b; }
	function _call_lt( $a, $b )	{ return $a < $b; }
	function _call_lteq( $a, $b )	{ return $a <= $b; }
	function _call_gt( $a, $b )	{ return $a > $b; }
	function _call_gteq( $a, $b )	{ return $a >= $b; }

	/*
	** Instrukcja warunkowa
	*/
	function _call_if( $if, $then, $else ) {
		return $if ? $then : $else;
	}

	/*
	** Warto domylna
	*/
	function _call_default( $variable, $default ) {
		return $variable != '' ? $variable : $default;
	}

	/*
	** Escapowanie stringw, from SMARTY
	*/
	function _call_escape($string, $esc_type = 'html') {
		switch ($esc_type) {
			case 'html': 		return htmlspecialchars($string, ENT_QUOTES);
			case 'htmlall':		return htmlentities($string, ENT_QUOTES);
			case 'url':		return urlencode($string);
			// escape unescaped single quotes
			case 'quotes': 		return preg_replace("%(?<!\\\\)'%", "\\'", $string);
			// escape quotes and backslashes and newlines
			case 'javascript':	return strtr($string, array('\\'=>'\\\\',"'"=>"\\'",'"'=>'\\"',"\r"=>'\\r',"\n"=>'\\n'));
			// escape every character into hex
			case 'hex': $return = '';
				for ($x=0; $x < strlen($string); $x++) {
					$return .= '%' . bin2hex($string[$x]);
				}
				return $return;
			case 'hexentity':
				$return = '';
				for ($x=0; $x < strlen($string); $x++) {
					$return .= '&#x' . bin2hex($string[$x]) . ';';
				}
				return $return;

			default:
				return $string;
		}
	}
		
	/*
	** Drukowanie opcji HTML
	*/
	function _call_html_options( $options, $selected, $with_keys ) {
		$ret = '';
		if ( ! is_array( $options ) ) return $ret;
		if ( $with_keys ) {
			foreach( $options as $k => $v ) {
				$ret .= '<option';
				$ret .= $k == $selected || ( is_array( $selected ) && in_array( $k, $selected ) ) ? ' selected' : '';
				$ret .= ' value="' . htmlspecialchars( $k, ENT_QUOTES ) . '">';
				$ret .= htmlspecialchars( $v, ENT_QUOTES );
				$ret .= '</option>';
			}
		} else {
			foreach( $options as $k => $v ) {
				$ret .= '<option';
				$ret .= $v == $selected || ( is_array( $selected ) && in_array( $v, $selected ) ) ? ' selected' : '';
				$ret .= '>';
				$ret .= htmlspecialchars( $v, ENT_QUOTES );
				$ret .= '</option>';
			}
		}
		return $ret;
	}

	/*
	** PHP Wrappers
	*/
	function _call_count( $array ) 		{ return count( $array ); }
	function _call_trim( $string ) 		{ return trim( $string ); }

}
