<?

/*
 * php-booba - simple PHP web application network
 * Copyright (C) 2004  Marcin Gajda
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Obsuga systemu ticketw.
 */
class TicketService
{
	var $tickets;
	var $ticket_groups;

	function TicketService()
	{
		$this->getTickets();
	}
	
	/*
	 * Zapisanie wszystkich ticketw do sesji.
	 */
	function saveTickets()
	{
		$_SESSION[ 'tickets' ] = $this->tickets;
	}

	/*
	 * Pobranie ticketw z sesji.
	 */
	function getTickets()
	{
		$this->tickets = isset( $_SESSION[ 'tickets' ] ) ? $_SESSION[ 'tickets' ] : array();
		$this->ticket_groups = isset( $_SESSION[ 'tickets' ] ) ? $_SESSION[ 'ticket_groups' ] : array();
	}

	/*
	 * Dodanie nowego ticketu.
	 *
	 * Zwracana jest warto ticketu. Mona poda grup ticketw, do ktrej
	 * ma by on dodany.
	 */
	function addTicket( $valuesToStore , $group = NULL )
	{
		$tid = $this->getNewTicketName();
		
		$this->tickets[ $tid ] = $valuesToStore;

		if( !is_null( $group ) )
		{
			if( !isset( $this->ticket_groups[ $group ] ) )
				$this->ticket_groups[ $group ] = array();

			$this->ticket_groups[ $group ][] = $tid;
		}

		return $tid;
	}

	/*
	 * Zwrcenie unikalnej nazwy ticketu (takiej,
	 * jakiej jeszcze nie ma.
	 */
	function getNewTicketName()
	{
		do
		{
			$tid = mhash( MCRYPT_SHA1 , time().rand( 65533 ) );
		}
		while( $this->isTicketSet( $tid ) );

		return $tid;
	}

	/*
	 * Sprawdzenie, czy dany ticket jest ustawiony.
	 */
	function isTicketSet( $tid )
	{
		return isset( $this->tickets[ $tid ] );
	}

	/*
	 * Pobranie ticketu.
	 */
	function getTicket( $tid )
	{
		return $this->isTicketSet( $tid ) ? $this->tickets[ $tid ] : NULL;
	}

	/*
	 * Usunicie wszystkich ticketw.
	 *
	 * Jeil podana grupa, to usuwane s jedynie tickety z tej grupy.
	 */
	function clear( $group = NULL )
	{
		if( is_null( $group ) )
			$this->tickets = array();
		else
			unset( $this->tickets[ $group ] );
	}

	/*
	 * Odnalezienie wybranego ticketu.
	 * Funkcja powinna by przeciana w klasach, aby mona
	 * byo na stae podefiniowa sobie pewne klasy podstron.
	 * Tutaj jest zdefiniowana jedynie klasa dla strony gwnej.
	 */
	function findTicket( $tid )
	{
		$request = $this->getTicket( $tid );

		return $request == NULL ? array() : $request;
	}
}

/*
 * Obiekt powinien by wsplny.
 */
function &getTicketService()
{
	global $_CMS_TICKET;

	if( !is_object( $_CMS_TICKET ) )
		$_CMS_TICKET = new TicketService();
	
	return $_CMS_TICKET;
}

?>
